<?php namespace Engine\Job\Publish\Handlers\Couch;

use Exception;
use Doctrine\CouchDB\CouchDBClient;

class CouchHandler
{

    /**
     * @var CouchDBClient
     */
    private $client;

    /**
     * CouchHandler constructor.
     * @param $host
     * @param $port
     * @param $database
     * @throws Exception
     */
    public function __construct($host, $port, $database, $type = "socket", $timeout = 10, $logging = false)
    {
        try {

            $options = [
                'ip' => $host,
                'port' => $port,
                'dbname' => $database,
                'type' => $type,  // stream|socket
                'timeout' => $timeout,
                'logging' => $logging
            ];

            // Create a couch client
            $this->client = CouchDBClient::create($options);

        } catch(Exception $e) {
            throw $e;
        }
    }

    /**
     * getCouchClient
     * @return CouchDBClient
     */
    public function getCouchDBClient()
    {
        return $this->client;
    }

    /**
     * @param $client
     * @param $document
     * @param $data
     * @return mixed
     */
    public function save($document, $data)
    {
        $res = $this->client->findDocument($document);


        if( $res->status == 200 )
        {
            $rev = $res->body['_rev'];

            $result = $this->client->putDocument(array('_rev' => $rev, "data" => $data), $document);
        }
        else
        {
            // If the document not exists it will be created
            //$result = $this->client->putDocument(array("data" => $data), $document);
            $result = $this->client->postDocument(array("data" => $data));
            // TODO Pode dar erro 400 de dados mal formados
        }

        return $result;
    }

}

