<?php namespace Engine\Job\Publish\Handlers\Couch\DesignDocuments;

class ChannelsByIsoByTag implements \Doctrine\CouchDB\View\DesignDocument
{
    private $orderBy;
    private $viewName;

    public function __construct($viewName = 'channels_by_iso_by_tag', $orderBy = 'position')
    {
        $this->orderBy = $orderBy;
        $this->viewName = $viewName;
    }

    public function getData()
    {
        return array(
            'language' => 'javascript',
            'views' => array(
                $this->viewName => array(
                    'map' => 'function(doc) {
                        if(\'channel\' == doc.type) {
                            if( doc.data.l10n.length ) {
                              for ( var j=0 ; j <= doc.data.l10n.length - 1; j++ ){
                                iso = doc.data.l10n[j][\'isoLanguage\'];
                                if( doc.data.l10n[j][\'tags\'].length ) {          
                                  for ( var i=0 ; i <= doc.data.l10n[j][\'tags\'].length - 1; i++ ){
                                  
                                  l10n = doc.data.l10n.filter(function(item){
                                      return item.isoLanguage == doc.data.l10n[j][\'isoLanguage\'];
                                    });

                				   tags = doc.tags.filter(function(item){
                                      return item.isoLanguage == doc.data.l10n[j][\'isoLanguage\'];
                                   });
                                   
                                   if( \'playlist\' in doc.data ) { 
                                       playlist = doc.data.playlist.l10n.filter(function(item){
                                        return item.isoLanguage == doc.data.l10n[j][\'isoLanguage\'];
                                      })
                                   } else {
                                      playlist = {};
                                   }
                                                           
                                    obj = {
                                      _id: doc._id,
                                      _rev: doc._rev,
                                      created_at: doc.created_at,
                                      type: doc.type,
                                      tags: tags.length ? tags[0][\'list\'] : {},
                                      data: {
                                        id: doc.data.id,
                                        l10n: l10n.length ? l10n[0] : {},
                                        name: doc.data.name,
                                        description: doc.data.description,
                                        domain: doc.data.domain,
                                        total: doc.data.total,
                                        date: {
                                          register: doc.data.date.register,
                                          end: doc.data.date.end,
				                          publish: doc.data.date.publish,
				                          begin: doc.data.date.begin,
                                        },
                                        playlists: {
                                          count: doc.data.playlists.count
                                        },
                                        /*playlist: playlist.length ? playlist[0] : {},*/
                                      },
                                      /*channelTags: doc.channelTags,*/
                                      /*videos: {
                                      count: doc.data.videos.count
                                      }*/
                                    };
                                    
                                    if( playlist.length ) {
                                        obj[\'data\'][\'playlist\'] = playlist[0];
                                    }
                                    tag = doc.data.l10n[j].tags[i][\'slug\'];
                                    name = obj.data.l10n[\'name\'];
                                    position = doc.data.position;
                                    emit([iso, tag, ' . $this->orderBy . '], obj)
                                  }
                                } 
                              }
                            }
                          }
                    }',
                    "reduce" => "_count"
                ),
            ),
        );
    }
}