<?php namespace Engine\Job\Publish;

use Stage\Account\Account;
use Stage\Account\Config\Config;
use Stage\Optimus;

abstract class Publish {

    const POSTS = "posts";
    const ASSETS = "assets";
    const VIDEO = "video";
    const PLAYER = "players";
    const ADS = "ads";
    const LIVE = "live";

    /**
     * Database that contains the access credentials, ie, apikeys and apisecrets
     */
    const CREDENTIALS_DATABASE = "credentials";

    /**
     * @var Account
     */
    protected $account;

    /**
     * @var
     */
    protected $service;

    /**
     * @var
     */
    protected $collections;

    /**
     * @var
     */
    protected $dossiers;

    /**
     * @var
     */
    protected $albums;

    /**
     * @var
     */
    protected $galleries;

    /**
     * @var
     */
    protected $assetsDirectories;

    /**
     * @var
     */
    protected $channels;

    /**
     * @var
     */
    protected $assets;

    /**
     * @var
     */
    protected $videoComposes;

    /**
     * @var Optimus
     */
    protected $optimus;

    /**
     * @var
     */
    private $playUrlHost;

    public function __construct(Account $account, $service = "")
    {
        $this->account = $account;

        $this->service = $service;
    }

    public function getAccount()
    {
        return $this->account;
    }

    public function getService()
    {
        return $this->service;
    }

    public function getCollections()
    {
        return $this->collections;
    }

    public function getDossiers()
    {
        return $this->dossiers;
    }

    public function getAlbums()
    {
        return $this->albums;
    }

    public function getGalleries()
    {
        return $this->galleries;
    }

    public function getChannels()
    {
        return $this->channels;
    }

    public function getOptimus()
    {
        return $this->optimus;
    }

    public function getPlayUrlHost()
    {
        return $this->playUrlHost;
    }

    public function getAssetsDirectories()
    {
        return $this->assetsDirectories;
    }

    public function getAssetBaseUrl()
    {
        $account = $this->getAccount();
        return $account->getConfig()->get(Config::ASSET_IMAGE) . DIRECTORY_SEPARATOR . $account->getToken();
    }

    public function getVideoBaseUrl()
    {
        $account = $this->getAccount();
        return $account->getConfig()->get(Config::VIDEO_IMAGE) . DIRECTORY_SEPARATOR . $account->getToken();
    }

    public function getAssets()
    {
        return $this->assets;
    }

    public function getVideoComposes()
    {
        return $this->videoComposes;
    }

    public static function getServiceCouchDBName($service, $idAccount, $suffix = null)
    {
        return $service . "_". strtolower($idAccount) . ( $suffix ? $suffix : "");
    }

    public function addCollections($collections)
    {
        $this->collections = $collections;
    }

    public function addAlbums($albums)
    {
        $this->albums = $albums;
    }

    public function addGalleries($galleries)
    {
        $this->galleries = $galleries;
    }

    public function addAssetsDirectories($directories)
    {
        $this->assetsDirectories = $directories;
    }

    public function addChannels($channels)
    {
        $this->channels = $channels;
    }

    public function addAssets($assets)
    {
        $this->assets = $assets;
    }

    public function addVideoComposes($videoComposes)
    {
        $this->videoComposes = $videoComposes;
    }

    public function setOptimus(Optimus $optimus)
    {
        $this->optimus = $optimus;
    }

    public function setPlayUrlHost($url)
    {
        $this->playUrlHost = $url;
    }

}