<?php namespace Engine\Job\Publish\Services\Ads;

use Engine\Job\Publish\Services\Ads\Schemas\CampaignSchema;
use Exception;
use Engine\Job\Publish\Handlers\Couch\CouchHandler;
use Engine\Job\Publish\Support\Ad as AdSupport;
use Stage\Account\Account;
use Stage\Account\Config\Config;
use Stage\Ad\Campaign\Campaign;

class Migrate {

    /**
     * @var Account
     */
    private $account;

    /**
     * @var CouchHandler
     */
    private $couchHandler;

    const TYPE_CAMPAIGN = 1;

    const ACTION_CREATE = 1;
    const ACTION_UPDATE = 2;
    const ACTION_DELETE = 3;

    public function __construct(Account $account, Config $config)
    {
        $this->account = $account;
        $this->config = $config;
    }

    public function setCouchHandler(CouchHandler $baseUrl)
    {
        $this->couchHandler = $baseUrl;
    }

    public function start($fresh = false)
    {

        $couchHandler = $this->couchHandler;

        // Get all the changed Posts
        $changed = AdSupport::changed();

//        $changed = [
//            ['type' => 1, 'action_type' => 1, 'id' => 87, 'id_sync' => "", 'id_rev_sync' => "", 'update_date' => "", 'register_date' => "" ],
//            ['type' => 1, 'action_type' => 1, 'id' => 86, 'id_sync' => "", 'id_rev_sync' => "", 'update_date' => "", 'register_date' => "" ]
//        ];

        foreach ($changed as $ch)
        {

            // This id could be a post id, or dossier ir, or collection id
            $id = $ch['id'];
            $type = $ch['type'];
            $idSync = $ch['id_sync'];
            $idRev = $ch['id_rev_sync'];
            $actionType = $ch['action_type'];
            $registerDate = $ch['register_date'];
            $updateDate = $ch['update_date'];

            $data = [];

            if( $actionType != self::ACTION_DELETE )
            {

                switch ($type) {
                    case self::TYPE_CAMPAIGN:

                        /** @var Campaign $campaign */
                        $campaign = Campaign::find($id);

                        // Create a new CampaignSchema
                        $campaignSchema = new CampaignSchema($this->account, $this->config, $campaign, $registerDate, $updateDate);

                        // Build it
                        $data = $campaignSchema->build();

                        break;
                }
            }

            switch ($actionType)
            {
                case self::ACTION_CREATE:

                    if( $data )
                    {
                        try {
                            // Save
                            list($id_, $rev_) = $this->couchHandler->getCouchDBClient()->postDocument($data);

                            // Sync with mysql database
                            AdSupport::sync($id, $id_, $rev_, $type);
                        } catch (Exception $e){
                            throw $e;
                        }
                    }
                    break;

                case self::ACTION_UPDATE:

                    // Check if the idSync is not empty
                    if( !empty($idSync) )
                    {
                        try {
                            // Update
                            list($id_, $rev_) = $this->couchHandler->getCouchDBClient()->putDocument($data, $idSync, $idRev);

                            // Sync with mysql database
                            AdSupport::sync($id, $id_, $rev_, $type);

                        } catch (Exception $e){
                            throw $e;
                        }
                    }

                    break;

                case self::ACTION_DELETE:

                    try {

                        $doc = $this->couchHandler->getCouchDBClient()->findRevisions($idSync);
                        if( $doc->status == 200 )
                        {
                            $body = $doc->body;
                            foreach ($body as $b)
                            {
                                $rev_ = $b['ok']['_rev'];
                                $this->couchHandler->getCouchDBClient()->deleteDocument($idSync, $rev_);

                            }
                        }

                    } catch (Exception $e){
                        throw $e;
                    }

                    break;

            }
        }

        // Create design documents if it the database is fresh
        if( $fresh )
        {
            $this->createDesignDocuments($couchHandler);
        }

        // Compact database
        // $couchHandler->getCouchDBClient()->compactDatabase();

    }

    /**
     * @param CouchHandler $couchHandler
     */
    public function createDesignDocuments(CouchHandler $couchHandler)
    {
        // _design/$designName
        $client = $couchHandler->getCouchDBClient();

        $designName = 'campaign_composed';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\CampaignComposed());

        $designName = 'all_docs';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\AllDocs());

    }

}
