<?php namespace Engine\Job\Publish\Services\Ads\Schemas;

use DateTime;
use Stage\Ad\Campaign\Campaign;
use Stage\Ad\Campaign\Group\Group;
use Stage\Account\Account;
use Stage\Account\Config\Config;
use Stage\Ad\Video\Video as AdVideo;

class CampaignSchema {

    /**
     * @var Campaign
     */
    private $campaign;

    /**
     * @var Config
     */
    private $config;

    /**
     * @var Account
     */
    private $account;

    const DATETIME_FORMAT = 'Y/m/d H:i:s O';

    public function __construct(Account $account, Config $config, Campaign $campaign, $registerDate, $updateDate)
    {
        $this->account = $account;
        $this->config = $config;
        $this->campaign = $campaign;
        $this->beginAt = (new DateTime($this->campaign->getBeginDate()))->format(self::DATETIME_FORMAT);
        $this->endAt = !$this->campaign->getEndDate() ? null : (new DateTime($this->campaign->getEndDate()))->format(self::DATETIME_FORMAT);
        $this->registerDate = (new DateTime($registerDate))->format(self::DATETIME_FORMAT);
        $this->updateDate = (new DateTime($updateDate))->format(self::DATETIME_FORMAT);
    }

    public function build()
    {

        $campaign = $this->campaign;

        $composesIds = $campaign->composes();

        $data = ['id' => $campaign->getId(),
            'data' => [
                'id' => (int) $campaign->getId(),
                'name' => $campaign->getName()
                //'l10n' => $l10n,   // Filtrar so os L10n's que estao no estado 5
            ],
            // 'dossiers' => $this->dossiers($dossiers),
            'groups' => $this->groups($campaign),
            'tags' => $this->tags($campaign),
            // 'iso' => $isos,
            'begin_at' => $this->beginAt,
            'end_at' => $this->endAt,
            'created_at' => $this->registerDate,
            'updated_at' => $this->updateDate,
            'hasAdVideos' => $campaign->hasAdVideos(),
            'hasExternalTag' => $campaign->hasExternalTag(),
            'on' => $this->on($campaign->getId(), $composesIds),
            'type' => "campaign"];

        return $data;
    }

    public function on($idCampaign, $composesIds)
    {
        $temp = [];
        foreach ($composesIds as $idCompose)
        {
            $temp[] = [(int)$idCompose, $idCampaign];
        }

        return $temp;
    }

    public function groups(Campaign $campaign)
    {
        $account = $this->account;
        $config = $this->config;

        // Get the Camppaign Groups
        $groups = $campaign->groups();

        // Convert the Campaign Groups to array
        $temp['groups'] = array_map(function($group){
            return $group->toArray();
        }, $groups);

        $allDevices = 'allDevices';
        $mobile = 'mobile';
        $web = 'web';

        for ( $k = 0 ; $k <= sizeof($temp['groups']) - 1 ; $k++ )
        {

            $g = $temp['groups'][$k];

            $videos = \Stage\Ad\Campaign\Group\Group::videos($g['campaign']['id'], $g['id']);
            /** @var \Stage\Ad\Campaign\Group\Ad\Video $v */
            for ( $i = 0 ; $i <= sizeof($videos) - 1 ; $i++ )
            {
                $v = $videos[$i];

                if( $v->displayIn() == Group::DISPLAY_IN_WEB )
                {
                    $key = $web;
                }
                else if( $v->displayIn() == Group::DISPLAY_IN_MOBILE )
                {
                    $key = $mobile;
                }
                else if( $v->displayIn() == Group::DISPLAY_IN_ALL )
                {
                    $key = $allDevices;
                }

                $t = $v->toArray();
                $t['beginDate'] =  (new DateTime($t['beginDate']))->format(self::DATETIME_FORMAT);
                $t['endDate'] = !$t['endDate'] ? null : (new DateTime($t['endDate']))->format(self::DATETIME_FORMAT);

                unset($t['id']);
                unset($t['idCampaignGroup']);
                unset($t['idCampaign']);
                unset($t['idAd']);
                unset($t['registerDate']);

                /** @var \Stage\Ad\Video\Video $adVideo */
                $adVideo = \Stage\Ad\Video\Video::find($v->getIdAd());
                $t['video'] =  $adVideo->toArray();

                $pieces = [$config->get(Config::ADS_CDN_SOURCE), $account->getToken(), $adVideo->getFilename()];
                $path = implode(DIRECTORY_SEPARATOR, $pieces);

                $t['video']['path'] = $path;
                $t['video']['versions'] = array_collapse($this->versions($adVideo));
                // $t['videoId'] = $adVideo->getId();
                $temp['groups'][$k][$key]['videos'][] = $t;

            }

            $adTags = \Stage\Ad\Campaign\Group\Group::adTags($g['campaign']['id'], $g['id']);
            /** @var \Stage\Ad\Campaign\Group\Ad\Tag $v */
            for ( $i = 0 ; $i <= sizeof($adTags) - 1 ; $i++ )
            {
                $v = $adTags[$i];

                $adTag = \Stage\Ad\External\Tag::find($v->getIdAdTag());

                if( $v->displayIn() == Group::DISPLAY_IN_WEB )
                {
                    $key = $web;
                }
                else if( $v->displayIn() == Group::DISPLAY_IN_MOBILE )
                {
                    $key = $mobile;
                }
                else if( $v->displayIn() == Group::DISPLAY_IN_ALL )
                {
                    $key = $allDevices;
                }

                $t = $adTag->toArray();
                unset($t['active']);

                $temp['groups'][$i][$key]['adTag'][] = $t;

            }

            $isos = [];
            $countries = \Stage\Ad\Campaign\Group\Group::countries($g['campaign']['id'], $g['id']);
            for ( $i = 0 ; $i <= sizeof($countries) - 1 ; $i++ )
            {
                if( $countries[$i]->getIso() != "" )
                    $isos[] = $countries[$i]->getIso();
            }

            unset($temp['groups'][$k]['campaign']);
            unset($temp['groups'][$k]['registerDate']);

            $temp['groups'][$k]['iso'] = $isos;

        }

        return $temp;
    }

    public function versions(AdVideo $video)
    {

        $cdn            = $this->config->get(Config::ADS_CDN_VERSIONS);
        $source         = $this->config->get(Config::STORAGE_ADS);
        $versionsPath   = $this->config->get(Config::ADS_VERSIONS_PATH);
        $accountToken   = $this->account->getToken();

        $ver = [];

        // Publish to couch all the versions
        $versions = [\Stage\Ad\Video\Version::WEB, \Stage\Ad\Video\Version::MOBILE, \Stage\Ad\Video\Version::LOW_MOBILE];
        foreach ($versions as $v)
        {

            $ver[] = array_map( function($version) use ($video, $cdn, $source, $accountToken, $versionsPath) {

                // Its the filename without extension
                $fileName = pathinfo($version->getFilename())['filename'];
                $versionType = $version->getVersionType();
                $versionNumber = $version->getVersion();
                $versionExtension = $version->getExtension();

                // Its the video url to the file
                $source = implode(DIRECTORY_SEPARATOR, [$cdn, $accountToken, $versionNumber, $versionType, $fileName]);

                // Its the video source on disk
                $file = implode(DIRECTORY_SEPARATOR, [$versionsPath, $accountToken, $versionNumber, $versionType, $fileName]);

                // Its the mime type
                $mimeType = file_exists($file . '.' . $version->getExtension()) ? mime_content_type($file . '.' . $version->getExtension()) : null;

                $temp['width'] = (int) $version->getWidth();
                $temp['height'] = (int )$version->getHeight();
                $temp['version'] = (int) $version->getVersion();
                $temp['versionType'] = $version->getVersionType();
                $temp['extension'] = $version->getExtension();
                $temp['isDefault'] = $version->getIsDefault();
                $temp['size'] = $version->getSize();
                $temp['mimeType'] = $mimeType;
                $temp['bitrate'] = (int) $version->getBitrate();
                $temp['source'] = $source . '.' . $version->getExtension();
                return $temp;

            }, $video->versions($v, 'version', SORT_ASC));

        }

        return $ver;
    }

    public function tags(Campaign $campaign)
    {
        $tags = $campaign->tags();
        return array_map(function($tag){
            $t = $tag->toArray();
            $t['slug'] = $tag->getNameStripped();
            unset($t['public']);
            unset($t['nameStripped']);
            unset($t['totalAds']);
            unset($t['totalCampaigns']);
            return $t;
        }, $tags);
    }
}