<?php namespace Engine\Job\Publish\Services\Assets;

use Engine\Job\Publish\Handlers\Couch\CouchHandler;
use Exception;
use Stage\Account\Account;
use Stage\Asset\Album\Album;
use Stage\Asset\Directory;
use Stage\Asset\Query;
use Stage\Asset\Gallery\Query\Album as AlbumQuery;

class Migrate {

    /**
     * @var Account
     */
    private $account;

    /**
     * @var
     */
    private $albums;

    /**
     * @var
     */
    private $galleries;

    /**
     * @var
     */
    private $directories;

    /**
     * @var
     */
    private $baseUrl;

    /**
     * @var
     */
    private $assets;

    /**
     * @var CouchHandler
     */
    private $couchHandler;

    /**
     * @var
     */
    private $viewsExist = false;

    /**
     * @var int
     *
     */
    private $maxIterations = 20000;

    public function __construct(Account $account, $albums, $galleries, $directories, $baseUrl, $assets)
    {
        $this->account = $account;
        $this->albums = $albums;
        $this->galleries = $galleries;
        $this->directories = $directories;
        $this->baseUrl = $baseUrl;
        $this->assets = $assets;
    }

    public function getAccount()
    {
        return $this->account;
    }

    public function getAlbums()
    {
        return $this->albums;
    }

    public function getGalleries()
    {
        return $this->galleries;
    }

    public function getDirectories()
    {
        return $this->directories;
    }

    public function getBaseUrl()
    {
        return $this->baseUrl;
    }

    public function getAssets()
    {
        return $this->assets;
    }

    public function setCouchHandler(CouchHandler $baseUrl)
    {
        $this->couchHandler = $baseUrl;
    }

    public function start()
    {

        $couchHandler = $this->couchHandler;

        // Delete old documents
        // $this->deleteOldDocuments($couchHandler);

        // Populate Couch
        $this->assets($couchHandler, $this->getAssets());
        $this->albums($couchHandler, $this->getAlbums());
        $this->albumsAssets($couchHandler, $this->getAlbums());
        $this->galleries($couchHandler, $this->getGalleries());
        $this->galleriesAlbums($couchHandler, $this->getGalleries());
        $this->galleriesAlbumsAssets($couchHandler, $this->getGalleries());
        $this->directories($couchHandler, $this->getDirectories());
        $this->directoriesAssets($couchHandler, $this->getDirectories());

        // Create design documents
        $this->createDesignDocuments($couchHandler);

        // Compact database
        $couchHandler->getCouchDBClient()->compactDatabase();

        // Generate the ETag last entry
        $this->generateETag($couchHandler);
    }

    /**
     * @param CouchHandler $couchHandler
     */
    public function deleteOldDocuments(CouchHandler $couchHandler)
    {

        $client = $couchHandler->getCouchDBClient();

        // Query all articles.
        $query = $client->allDocs();

        foreach ($query->body['rows'] as $document)
        {

            $idDocument = $document['id'];
            $rev = $document['doc']['_rev'];

            // Delete a single document.
            if( !isset($document['doc']['views']) )
                $client->deleteDocument($idDocument, $rev);
            else
                $this->viewsExist = true;

        }

    }

    /**
     * @param CouchHandler $couchHandler
     */
    public function createDesignDocuments(CouchHandler $couchHandler)
    {
        // _design/$designName
        $client = $couchHandler->getCouchDBClient();

        $designName = 'assets';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\Assets());

        $designName = 'albums';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\Albums());

        $designName = 'albums';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\AlbumsByName());

        $designName = 'albums_assets';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\AlbumsAssets());

        $designName = 'galleries';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\Galleries());

        $designName = 'galleries_albums';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\GalleriesAlbums());

        $designName = 'galleries_albums_assets';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\GalleriesAlbumsAssets());

        $designName = 'directories';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\AssetsDirectories());

        $designName = 'directory_assets';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\AssetsDirectoriesAssets());

        $designName = 'all_docs';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\AllDocs());

        $designName = 'etag';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ETag());

    }

    /**
     * @param CouchHandler $couchHandler
     * @param $assets
     */
    public function assets(CouchHandler $couchHandler, $assets)
    {
        $bulk = [];
        $i = 0;
        foreach ($assets as $asset)
        {

            $data = $asset->toArray();
            unset($data['state']);
            $data['baseUrl'] = $this->getBaseUrl();

            $bulk[] = ["data" => $data, "type" => "asset"];

            if( $i >= $this->maxIterations )
            {
                $bulkUpdater = $couchHandler->getCouchDBClient()->createBulkUpdater();
                $bulkUpdater->updateDocuments($bulk);
                $response = $bulkUpdater->execute();

                // Reset the counters
                $i = 0;
                $bulk = [];
            }

            $i ++;

            // Save on couch
            // $couchHandler->getCouchDBClient()->postDocument(["data" => $data, "type" => "asset"]);

        }

        $bulkUpdater = $couchHandler->getCouchDBClient()->createBulkUpdater();
        $bulkUpdater->updateDocuments($bulk);
        $response = $bulkUpdater->execute();
    }

    /**
     * dossiers
     *
     * @param $client
     */
    public function albums(CouchHandler $couchHandler, $albums)
    {

        $bulk = [];
        $i = 0;
        foreach ($albums as $album)
        {

            $data = $album->toArray();

            $data['assets'] = ['count' => $data['totalImages']];
            $data['cover'] = ['baseUrl'=> $this->getBaseUrl(), 'photo' => $data['coverImage']];

            unset($data['totalImages']);
            unset($data['coverImage']);

            $data['galleries'] = [];
            try {

                $galleries = $album->associatedGalleries();
                if( $galleries )
                {
                    $data['galleries'] = $galleries;
                }

            } catch (Exception $e){ }

            $bulk[] = ["data" => $data, "type" => "album"];
            if( $i >= $this->maxIterations )
            {
                $bulkUpdater = $couchHandler->getCouchDBClient()->createBulkUpdater();
                $bulkUpdater->updateDocuments($bulk);
                $response = $bulkUpdater->execute();

                // Reset the counters
                $i = 0;
                $bulk = [];
            }

            $i ++;

            // Save on couch
            // $couchHandler->getCouchDBClient()->postDocument(["data" => $data, "type" => "album"]);
        }

        $bulkUpdater = $couchHandler->getCouchDBClient()->createBulkUpdater();
        $bulkUpdater->updateDocuments($bulk);
        $response = $bulkUpdater->execute();

    }

    /**
     * @param CouchHandler $couchHandler
     * @param $albums
     */
    public function albumsAssets(CouchHandler $couchHandler, $albums)
    {

        $data = [];
        foreach ($albums as $album)
        {

            $query = new Query();
            $query->term    = "";
            $query->from    = 0;
            $query->to      = -1;
            $query->filter  = "";
            $query->by      = "";

            $i = 0;
            $bulk = [];
            $assets = $album->assets($query);
            foreach ($assets as $asset)
            {
                $data = $asset->toArray();
                unset($data['state']);
                unset($data['rooms']);
                $data['baseUrl'] = $this->getBaseUrl();

                $bulk[] = ["data" => $data, "type" => "album-asset", "album" => ['id' => $album->getId()]];
                if( $i >= $this->maxIterations )
                {
                    $bulkUpdater = $couchHandler->getCouchDBClient()->createBulkUpdater();
                    $bulkUpdater->updateDocuments($bulk);
                    $response = $bulkUpdater->execute();

                    // Reset the counters
                    $i = 0;
                    $bulk = [];
                }

                $i ++;

                // Save on couch
                // $couchHandler->getCouchDBClient()->postDocument(["data" => $data, "type" => "album-asset", "album" => ['id' => $album->getId()]]);

            }

            $bulkUpdater = $couchHandler->getCouchDBClient()->createBulkUpdater();
            $bulkUpdater->updateDocuments($bulk);
            $response = $bulkUpdater->execute();

        }

    }

    /**
     * @param $client
     */
    public function galleries(CouchHandler $couchHandler, $galleries)
    {

        $i = 0;
        $bulk = [];
        foreach ($galleries as $gallery)
        {
            $data = $gallery->toArray();
            if( !($gallery->getAlbum() instanceof Album) )
                unset($data['album']);

            $query = new AlbumQuery();
            $query->term = "";
            $query->from = 0;
            $query->to   = -1;
            $albums = $gallery->albums($query);

            $data['albums']['count'] = sizeof($albums);

            $tags = $tags = \Engine\Job\Publish\Support\Asset::assetsGalleriesTags($gallery->getId());
            foreach ($tags as $tag){
                $data['tags'][] = self::tags($tag);
            }

            $bulk[] = ["data" => $data, "type" => "gallery"];
            if( $i >= $this->maxIterations )
            {
                $bulkUpdater = $couchHandler->getCouchDBClient()->createBulkUpdater();
                $bulkUpdater->updateDocuments($bulk);
                $response = $bulkUpdater->execute();

                // Reset the counters
                $i = 0;
                $bulk = [];
            }

            $i ++;
            // Save on couch
            // $couchHandler->getCouchDBClient()->postDocument(["data" => $data, "type" => "gallery"]);
        }

        $bulkUpdater = $couchHandler->getCouchDBClient()->createBulkUpdater();
        $bulkUpdater->updateDocuments($bulk);
        $response = $bulkUpdater->execute();

    }

    /**
     * @param $client
     */
    public function galleriesAlbums(CouchHandler $couchHandler, $galleries)
    {

        $data = [];
        foreach ($galleries as $gallery)
        {

            $query = new AlbumQuery();
            $query->term = "";
            $query->from = 0;
            $query->to   = -1;
            $query->order = 2;
            $query->orderBy = 2;
            $albums = $gallery->albums($query);

            $i = 0;
            $bulk = [];
            foreach ($albums as $album)
            {
                $data = $album->toArray();

                $data['assets'] = ['count' => $data['totalImages']];
                if( $data['coverImage'] )
                    $data['cover'] = ['baseUrl'=> $this->getBaseUrl(), 'photo' => $data['coverImage']];

                unset($data['totalImages']);
                unset($data['coverImage']);

                $tags = $tags = \Engine\Job\Publish\Support\Asset::assetsAlbumsTags($album->getId());
                foreach ($tags as $tag){
                    $data['tags'][] = self::tags($tag);
                }

                $bulk[] = ["data" => $data, "type" => "gallery-album", "gallery" => ['id' => $gallery->getId()]];

                if( $i >= $this->maxIterations )
                {
                    $bulkUpdater = $couchHandler->getCouchDBClient()->createBulkUpdater();
                    $bulkUpdater->updateDocuments($bulk);
                    $response = $bulkUpdater->execute();

                    // Reset the counters
                    $i = 0;
                    $bulk = [];
                }

                $i ++;

                // Save on couch
                // $couchHandler->getCouchDBClient()->postDocument(["data" => $data, "type" => "gallery-album", "gallery" => ['id' => $gallery->getId()]]);

            }

            $bulkUpdater = $couchHandler->getCouchDBClient()->createBulkUpdater();
            $bulkUpdater->updateDocuments($bulk);
            $response = $bulkUpdater->execute();

        }

    }

    /**
     * @param CouchHandler $couchHandler
     * @param $galleries
     */
    public function galleriesAlbumsAssets(CouchHandler $couchHandler, $galleries)
    {

        $data = [];
        foreach ($galleries as $gallery)
        {

            $query = new AlbumQuery();
            $query->term = "";
            $query->from = 0;
            $query->to   = -1;
            $albums = $gallery->albums($query);

            foreach ($albums as $album)
            {
                $query = new Query();
                $query->term    = "";
                $query->from    = 0;
                $query->to      = -1;
                $query->filter  = "";
                $query->by      = "";

                $assets = $album->assets($query);

                $i = 0;
                $bulk = [];
                foreach ($assets as $asset)
                {
                    $data = $asset->toArray();

                    $q = new Query();
                    $q->idAsset = $asset->getId();
                    $tags = $asset->tags($q);

                    $tags = array_map(function ($tag) {
                        return $tag->toArray();
                    }, $tags);

                    $data['tags'] = $tags;
                    $data['baseUrl'] = $this->getBaseUrl();

                    $bulk[] = ["data" => $data,
                               "type" => "gallery-album-asset",
                               "album" => ['id' => $album->getId()],
                               "gallery" => ['id' => $gallery->getId()]];

                    if( $i >= $this->maxIterations )
                    {
                        $bulkUpdater = $couchHandler->getCouchDBClient()->createBulkUpdater();
                        $bulkUpdater->updateDocuments($bulk);
                        $response = $bulkUpdater->execute();

                        // Reset the counters
                        $i = 0;
                        $bulk = [];
                    }

                    $i ++;

                    // Save on couch
//                    $couchHandler->getCouchDBClient()->postDocument(["data" => $data, "type" => "gallery-album-asset",
//                                                                    "album" => ['id' => $album->getId()],
//                                                                    "gallery" => ['id' => $gallery->getId()]]);
                }

                $bulkUpdater = $couchHandler->getCouchDBClient()->createBulkUpdater();
                $bulkUpdater->updateDocuments($bulk);
                $response = $bulkUpdater->execute();

            }
        }
    }

    /**
     * @param CouchHandler $couchHandler
     * @param $galleries
     */
    public function directories(CouchHandler $couchHandler, $directories)
    {
        // Put the Root directory

        $rootDirectory = \Stage\Asset\Directory::find(\Stage\Asset\Directory::ROOT);

        array_unshift($directories, $rootDirectory);

        $i = 0;
        $bulk = [];
        foreach ($directories as $directory)
        {

            if( $directory instanceof \Stage\Asset\Directory ) {

                $data = $directory->toArray();

                $bulk[] = ["data" => $data, "type" => "directory"];
                if( $i >= $this->maxIterations )
                {
                    $bulkUpdater = $couchHandler->getCouchDBClient()->createBulkUpdater();
                    $bulkUpdater->updateDocuments($bulk);
                    $response = $bulkUpdater->execute();

                    // Reset the counters
                    $i = 0;
                    $bulk = [];
                }

                $i ++;

                // Save on couch
                // $couchHandler->getCouchDBClient()->postDocument(["data" => $data, "type" => "directory"]);
            }

        }

        $bulkUpdater = $couchHandler->getCouchDBClient()->createBulkUpdater();
        $bulkUpdater->updateDocuments($bulk);
        $response = $bulkUpdater->execute();
    }

    /**
     * @param CouchHandler $couchHandler
     * @param $directories
     */
    public function directoriesAssets(CouchHandler $couchHandler, $directories)
    {

        // Put the Root directory
        array_unshift($directories, \Stage\Asset\Directory::find(\Stage\Asset\Directory::ROOT));

        foreach ($directories as $directory)
        {
            if( $directory instanceof \Stage\Asset\Directory ) {

                $d = Directory::find($directory->getId());

                if( $d instanceof \Stage\Asset\Directory ) {
                    $query = new Query();
                    $query->by = Query::ALL;
                    $query->from = 0;
                    $query->to = -1;

                    $i = 0;
                    $bulk = [];
                    // Get all assets
                    $assets = Directory::assets($query, $d->getId());
                    foreach ($assets as $asset) {
                        $data = $asset->toArray();

                        $q = new Query();
                        $q->idAsset = $asset->getId();
                        $tags = $asset->tags($q);

                        $tags = array_map(function ($tag) {
                            return $tag->toArray();
                        }, $tags);


                        unset($data['state']);
                        unset($data['directoryId']);
                        unset($data['rooms']);
                        $data['tags'] = $tags;
                        $data['baseUrl'] = $this->getBaseUrl();

                        $bulk[] = ["data" => $data, "type" => "directory-asset", "directory" => ['id' => $d->getId()]];
                        if( $i >= $this->maxIterations )
                        {
                            $bulkUpdater = $couchHandler->getCouchDBClient()->createBulkUpdater();
                            $bulkUpdater->updateDocuments($bulk);
                            $response = $bulkUpdater->execute();

                            // Reset the counters
                            $i = 0;
                            $bulk = [];
                        }

                        $i ++;

                        // Save on couch
                        // $couchHandler->getCouchDBClient()->postDocument(["data" => $data, "type" => "directory-asset", "directory" => ['id' => $d->getId()]]);

                    }

                    $bulkUpdater = $couchHandler->getCouchDBClient()->createBulkUpdater();
                    $bulkUpdater->updateDocuments($bulk);
                    $response = $bulkUpdater->execute();
                }
            }
        }
    }

    /**
     * @param CouchHandler $couchHandler
     */
    public function generateETag(CouchHandler $couchHandler)
    {

        // Galleries
        $data = [];
        $galleries = $this->getGalleries();
        foreach ($galleries as $gallery)
        {
            $query = new AlbumQuery();
            $query->term = "";
            $query->from = 0;
            $query->to   = -1;
            $albums = $gallery->albums($query);

            $temp = [];
            foreach ($albums as $album)
            {
                $totalAssets = $album->getTotalImages();
                $temp[] = ['id' => $album->getId(), 'count' => $totalAssets];
            }

            $data[] = ['id' => (int) $gallery->getId(), 'count' => sizeof($albums), 'albums' => $temp];
        }

        // Albums
        $albumsData = [];
        $albums = $this->getAlbums();
        foreach ($albums as $album)
        {
            $albumsData[] = ['id' => (int) $album->getId(), 'assets' => ['count' => $album->getTotalImages()]];
        }

        // Directories
        $directories = $this->getDirectories();

        // Put the Root directory
        array_unshift($directories, \Stage\Asset\Directory::find(\Stage\Asset\Directory::ROOT));

        $directoriesData = [];
        foreach ($directories as $directory)
        {

            //$directory = Directory::find($directory->getId());
            if( $directory instanceof \Stage\Asset\Directory ) {
                $query = new Query();
                $query->by = Query::ALL;
                $query->from = 0;
                $query->to = -1;

                // Get all assets
                $assets = Directory::assets($query, $directory->getId());

                $directoriesData[] = ['id' => (int)$directory->getId(), 'count' => sizeof($assets)];
            }
        }

        $document = 'etag';
        $couchHandler->getCouchDBClient()->postDocument(["data" => $data,
                                                        "type" => $document,
                                                        "albums" => $albumsData ,
                                                        "directories" => $directoriesData,
                                                        "hash" => md5(time()) ]);
    }

    public static function tags($tag)
    {
        return ['id' => (int)$tag['id_tag'],
                'name' => $tag['name'],
                'slug' => $tag['name_stripped'],
                'count' => (int) $tag['total']];
    }
}
