<?php namespace Engine\Job\Publish\Services\Posts\Schemas;

use DateTime;
use Stage\Account\Account;
use Stage\Account\Config\Config;
use Stage\Post\Dossier\Dossier;
use Stage\Post\Post;

class DossierSchema {

    /**
     * @var Dossier
     */
    public $dossier;

    const DATETIME_FORMAT = 'Y/m/d H:i:s O';

    public function __construct(Account $account, Config $config, Dossier $dossier, $registerDate, $updateDate)
    {
        $this->account = $account;
        $this->config = $config;
        $this->dossier = $dossier;
        $this->registerDate = (new DateTime($registerDate))->format(self::DATETIME_FORMAT);
        $this->updateDate = (new DateTime($updateDate))->format(self::DATETIME_FORMAT);
        $this->beginDate = (new DateTime($dossier->getBeginDate()))->format(self::DATETIME_FORMAT);
        $this->endDate = (new DateTime($dossier->getEndDate()))->format(self::DATETIME_FORMAT);
    }

    public function build()
    {
        $allIsoLanguages = $this->allIsoLanguages();

        $data = [
            'id' => (int) $this->dossier->getId(),
            'name' => $this->dossier->getName(),
            'random' => $this->dossier->getRandom(),
            'tags' => $this->tags($allIsoLanguages),    // Esta key sao todas as diferentes tags de todos os posts dentro deste dossier
            'l10n' => $this->l10n(),
            'slug' => $this->slug(),
            'posts' => ['count' => $this->totalPosts()]
        ];

        return ["id" => $this->dossier->getId(),
                "data" => $data,
                "begin_at" => $this->beginDate,
                "end_at" => $this->endDate,
                "created_at" => $this->registerDate,
                "updated_at" => $this->updateDate,
                "on" => $this->on(), // Array of collections where this dossier exists
                "type" => "dossier"];
    }

    public function on()
    {
        return array_map(function($collection){
            return $collection->getId();
        },$this->dossier->collections());
    }

    public function allIsoLanguages()
    {
        // Get all iso languages of all Posts
        return Post::getAllIsoLanguages();
    }

    public function tags($isoLanguages)
    {

        $temp = [];
        foreach ($isoLanguages as $iso)
        {
            $key = $iso['iso_language'];
            $tags = \Engine\Job\Publish\Support\Post::postsDossiersTags($this->dossier->getId(), $key);

            foreach ($tags as $tag)
            {
                $temp[] = [
                    'id' => (int)$tag['id_tag'],
                    'name' => $tag['name'],
                    'slug' => $tag['name_stripped'],
                    'count' => (int) $tag['total']  // Count is the total number of times which the tag is used
                ];
            }
        }

        return $temp;
    }

    public function l10n()
    {
        return array_map(function($l10n){
            return $l10n->toArray();
            }, $this->dossier->getL10n());
    }

    public function slug()
    {
        return str_slug(html_entity_decode($this->dossier->getName(), ENT_COMPAT, "UTF-8"), "-");
    }

    public function totalPosts()
    {
        return (int) $this->dossier->totalPosts();
    }
}