<?php namespace Engine\Job\Publish\Services\Videos;

use Engine\Job\Publish\Services\Videos\Schemas\ChannelSchema;
use Engine\Job\Publish\Services\Videos\Schemas\ComposeSchema;
use Engine\Job\Publish\Services\Videos\Schemas\PlaylistSchema;
use Exception;
use Illuminate\Http\Response;
use Stage\Account\Account;
use Stage\Account\Config\Config;
use Engine\Job\Publish\Support\Video as VideoSupport;
use Stage\Video\Channel\Channel;
use Stage\Video\Compose\Compose as VideoCompose;
use Stage\Live\Compose\Compose as LiveCompose;
use Stage\Video\Playlist\Playlist;
use Engine\Job\Publish\Handlers\Couch\CouchHandler;

class Migrate extends \Engine\Job\Publish\Services\Migrate {

    /**
     * @var Account
     */
    private $account;

    /**
     * @var CouchHandler
     */
    private $couchHandler;

    /**
     * @var Config
     */
    private $config;

    /**
     * @var int
     */
    private $type;

    /**
     * @var \Illuminate\Support\Facades\Log
     */
    private $logger;

    public function __construct(Account $account, Config $config, $type)
    {
        $this->account = $account;
        $this->config = $config;
        $this->type = $type;
    }

    public function setCouchHandler(CouchHandler $baseUrl)
    {
        $this->couchHandler = $baseUrl;
    }

    public function setLogger(\Illuminate\Support\Facades\Log $logger = null)
    {
        $this->logger = $logger;
    }

    public function logToSlack($msg)
    {
        try {

            $data = new \stdClass();
            $attachemnt = new \stdClass();
            $attachemnt->title = "Exception";
            $attachemnt->pretext = "Publish to CouchDB Error";
            $attachemnt->color = "#CD2553"; // Green = #2eb886
            $attachemnt->text = $msg;
            $field = new \stdClass();
            $field->title = "Account";
            $field->value = $this->account->getCompanyName() . " ( " . $this->account->getId() . " )";
            $field->short = false;
            $field2 = new \stdClass();
            $field2->title = "Level";
            $field2->value = "Error";
            $field2->short = false;
            $attachemnt->fields = [$field];
            $data->attachments = [$attachemnt];

            $ch = curl_init();

            curl_setopt($ch, CURLOPT_URL, 'https://hooks.slack.com/services/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');
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
            curl_setopt($ch, CURLOPT_POST, 1);

            $headers = array();
            $headers[] = 'Content-Type: application/json';
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

            $result = curl_exec($ch);
            if (curl_errno($ch)) {
                echo 'Error:' . curl_error($ch);
            }
            curl_close($ch);

        } catch (\Exception $e){ }
    }

    public function start($fresh = false)
    {

        $couchHandler = $this->couchHandler;

        // Get all the changed Posts
        $changed = VideoSupport::changed($this->type);

        $logger = $this->logger;

//        $changed = [
//            ['type' => 3, 'action_type' => 1, 'id' => 1, 'id_sync' => "", 'id_rev_sync' => "", 'update_date' => "", 'register_date' => "2018-10-03 14:43:11", 'is_live' => 0 ],    // Channel Create
//            ['type' => 3, 'action_type' => 1, 'id' => 2, 'id_sync' => "", 'id_rev_sync' => "", 'update_date' => "", 'register_date' => "2018-10-03 14:43:11", 'is_live' => 0 ],    // Channel Create
//            ['type' => 3, 'action_type' => 1, 'id' => 3, 'id_sync' => "", 'id_rev_sync' => "", 'update_date' => "", 'register_date' => "2018-10-03 14:43:11", 'is_live' => 0 ],    // Channel Create
//            ['type' => 2, 'action_type' => 1, 'id' => 1, 'id_sync' => "", 'id_rev_sync' => "", 'update_date' => "", 'register_date' => "2018-10-03 14:43:11", 'is_live' => 0 ],    // Playlist Create
//            ['type' => 2, 'action_type' => 1, 'id' => 2, 'id_sync' => "", 'id_rev_sync' => "", 'update_date' => "", 'register_date' => "2018-10-03 14:43:11", 'is_live' => 0 ],    // Playlist Create
//            ['type' => 2, 'action_type' => 1, 'id' => 3, 'id_sync' => "", 'id_rev_sync' => "", 'update_date' => "", 'register_date' => "2018-10-03 14:43:11", 'is_live' => 0 ],    // Playlist Create
//            ['type' => 1, 'action_type' => 1, 'id' => 12, 'id_sync' => "", 'id_rev_sync' => "", 'update_date' => "", 'register_date' => "2018-10-03 14:43:11", 'is_live' => true],    // Composed Create
//            ['type' => 1, 'action_type' => 1, 'id' => 13, 'id_sync' => "", 'id_rev_sync' => "", 'update_date' => "", 'register_date' => "2018-10-03 14:43:11", 'is_live' => false ],     // Composed Create
//            ['type' => 1, 'action_type' => 1, 'id' => 14, 'id_sync' => "", 'id_rev_sync' => "", 'update_date' => "", 'register_date' => "2018-10-03 14:43:11", 'is_live' => false ],     // Composed Create
//            ['type' => 1, 'action_type' => 1, 'id' => 15, 'id_sync' => "", 'id_rev_sync' => "", 'update_date' => "", 'register_date' => "2018-10-03 14:43:11", 'is_live' => false ],    // Composed Create
//            ['type' => 1, 'action_type' => 1, 'id' => 16, 'id_sync' => "", 'id_rev_sync' => "", 'update_date' => "", 'register_date' => "2018-10-03 14:43:11", 'is_live' => false ],     // Composed Create
//            ['type' => 1, 'action_type' => 1, 'id' => 17, 'id_sync' => "", 'id_rev_sync' => "", 'update_date' => "", 'register_date' => "2018-10-03 14:43:11", 'is_live' => false ],     // Composed Create
//            ['type' => 1, 'action_type' => 1, 'id' => 18, 'id_sync' => "", 'id_rev_sync' => "", 'update_date' => "", 'register_date' => "2018-10-03 14:43:11", 'is_live' => false ],    // Composed Create
//
//            ['type' => 10, 'action_type' => 1, 'id' => 12, 'id_sync_player' => "", 'id_rev_sync_player' => "", 'update_date' => "", 'register_date' => "2018-10-03 14:43:11", 'is_live' => true ],    // Composed Create
//            ['type' => 10, 'action_type' => 1, 'id' => 13, 'id_sync_player' => "", 'id_rev_sync_player' => "", 'update_date' => "", 'register_date' => "2018-10-03 14:43:11", 'is_live' => false ],     // Composed Create
//            ['type' => 10, 'action_type' => 1, 'id' => 14, 'id_sync_player' => "", 'id_rev_sync_player' => "", 'update_date' => "", 'register_date' => "2018-10-03 14:43:11", 'is_live' => false ],     // Composed Create
//            ['type' => 10, 'action_type' => 1, 'id' => 15, 'id_sync_player' => "", 'id_rev_sync_player' => "", 'update_date' => "", 'register_date' => "2018-10-03 14:43:11", 'is_live' => false ],    // Composed Create
//            ['type' => 10, 'action_type' => 1, 'id' => 16, 'id_sync_player' => "", 'id_rev_sync_player' => "", 'update_date' => "", 'register_date' => "2018-10-03 14:43:11", 'is_live' => false ],     // Composed Create
//            ['type' => 10, 'action_type' => 1, 'id' => 17, 'id_sync_player' => "", 'id_rev_sync_player' => "", 'update_date' => "", 'register_date' => "2018-10-03 14:43:11", 'is_live' => false ],     // Composed Create
//            ['type' => 10, 'action_type' => 1, 'id' => 18, 'id_sync_player' => "", 'id_rev_sync_player' => "", 'update_date' => "", 'register_date' => "2018-10-03 14:43:11", 'is_live' => false ],    // Composed Create
//        ];

        foreach ($changed as $ch)
        {

            // This id could be a post id, or dossier ir, or collection id
            $id = $ch['id'];
            $type = $ch['type'];
            $idSync = $ch[$this->type === 1 ? 'id_sync' : 'id_sync_player'];
            $idRev = $ch[$this->type === 1 ? 'id_rev_sync' : 'id_rev_sync_player'];
            $actionType = $ch['action_type'];
            $registerDate = $ch['register_date'];
            $updateDate = $ch['update_date'];
            $isLive = (bool) $ch['is_live'];

            $format = '  Changed: Id "%s" Type "%s" IdSync "%s" IdRev "%s" ActionType "%s" RegisterDate "%s" UpdateDate "%s" IsLive "%s" => IdAccount: %s (%s)';
            $fullMsg = sprintf($format, $id, $type, $idSync, $idRev, $actionType, $registerDate, $updateDate, $isLive, $this->account->getId() , $this->account->getCompanyName());

            $data = [];

            if( $actionType != parent::ACTION_DELETE )
            {

                switch ($type) {
                    case parent::TYPE_CHANNEL:
                        try {
                            /** @var Channel $channel */
                            $channel = Channel::find($id);

                            // Create a new ChannelSchema
                            $channelSchema = new ChannelSchema($this->account, $this->config, $channel, $registerDate, $updateDate);

                            // Build it
                            $data = $channelSchema->build();
                        } catch (Exception $e) {
                            $msg  = "Exception: " . $e->getMessage();
                            $msg .= $fullMsg;
                            $this->logToSlack($msg);
                            $logger::error($msg);
                        }

                        break;

                    case parent::TYPE_PLAYLIST:

                        try {

                            /** @var Playlist $playlist */
                            $playlist = Playlist::find($id);

                            // Create a new PlaylistSchema
                            $playlistSchema = new PlaylistSchema($this->account, $this->config, $playlist, $registerDate, $updateDate);

                            // Build it
                            $data = $playlistSchema->build();

                        } catch (Exception $e) {
                            $msg  = "Exception: " . $e->getMessage();
                            $msg .= $fullMsg;
                            $this->logToSlack($msg);
                            $logger::error($msg);
                        }

                        break;

                    case parent::TYPE_COMPOSED_VIDEO:
                    case parent::TYPE_COMPOSED_VIDEO_PLAYER:

                        try {

                            $composedVideo = $isLive ? LiveCompose::find($id) : VideoCompose::find($id);
                            // Create a new ComposedVideoSchema
                            $composedVideoSchema = new ComposeSchema($this->account, $this->config, $composedVideo, $registerDate, $updateDate);

                            // Build it
                            $data = $composedVideoSchema->build();

                        } catch (Exception $e) {
                            $msg  = "Exception: " . $e->getMessage();
                            $msg .= $fullMsg;
                            $this->logToSlack($msg);
                            $logger::error($msg);
                        }

                        break;

                }
            }

            switch ($actionType)
            {
                case parent::ACTION_CREATE:

                    if( $data )
                    {
                        try {
                            // Save
                            list($id_, $rev_) = $this->couchHandler->getCouchDBClient()->postDocument($data);

                            // Sync with mysql database
                            VideoSupport::sync($id, $id_, $rev_, $type);
                        } catch (Exception $e){
                            $msg  = "Exception: " . $e->getMessage();
                            $msg .= $fullMsg;
                            $msg .= " | POST Document";
                            $this->logToSlack($msg);
                            $logger::error($msg);
                        }
                    }
                    break;

                case parent::ACTION_UPDATE:

                    // Check if the idSync is not empty
                    if( !empty($idSync) )
                    {
                        try {

                            $response = $this->couchHandler->getCouchDBClient()->findDocument($idSync);

                            if( $response->status == Response::HTTP_OK )
                            {
                                $body = $response->body;

                                $idRev = $body['_rev'];

                                // Update
                                list($id_, $rev_) = $this->couchHandler->getCouchDBClient()->putDocument($data, $idSync, $idRev);

                                // Sync with mysql database
                                VideoSupport::sync($id, $id_, $rev_, $type);
                            }
                            else
                            {
                                $msg = $fullMsg;
                                $msg .= " | PUT Document Not Found";
                                $this->logToSlack($msg);
                                $logger::error($msg);
                            }

                        } catch (Exception $e){
                            $msg  = "Exception: " . $e->getMessage();
                            $msg .= $fullMsg;
                            $msg .= " | PUT Document";
                            $this->logToSlack($msg);
                            $logger::error($msg);
                        }
                    }

                    break;

                case parent::ACTION_DELETE:

                    try {

                        $doc = $this->couchHandler->getCouchDBClient()->findRevisions($idSync);
                        if( $doc->status == Response::HTTP_OK )
                        {
                            $body = $doc->body;
                            foreach ($body as $b)
                            {
                                $rev_ = $b['ok']['_rev'];
                                $this->couchHandler->getCouchDBClient()->deleteDocument($idSync, $rev_);
                            }

                        }

                    } catch (Exception $e){
                        $msg  = "Exception: " . $e->getMessage();
                        $msg .= $fullMsg;
                        $msg .= " | DELETE Document";
                        $this->logToSlack($msg);
                        $logger::error($msg);
                    }

                    break;

            }
        }

        // Create design documents if it the database is fresh
        if( $fresh )
        {
            $this->createDesignDocuments($couchHandler);
        }

        // Compact database
        // $couchHandler->getCouchDBClient()->compactDatabase();

    }

    /**
     * createDesignDocuments
     *
     * @param CouchHandler $couchHandler
     */
    public function createDesignDocuments(CouchHandler $couchHandler)
    {
        // _design/$designName
        $client = $couchHandler->getCouchDBClient();

        // Player views
        if( $this->type == 2 )
        {
            $designName = 'simple_composed_video';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\SimpleComposedVideo());

            $designName = 'simple_composed_video_by_iso';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\SimpleComposedVideoByIso());

            $designName = 'all_docs';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\AllDocs());
        }
        else
        {
            $designName = 'simple_composed_video';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\SimpleComposedVideo());

            $designName = 'simple_composed_video_by_iso';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\SimpleComposedVideoByIso());

            $designName = 'channels';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\Channels());

            $designName = 'channels_by_position';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsByPosition());

            // This is default order is position
            $designName = 'channels_by_iso_by_tag';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsByIsoByTag($designName));

            $orderBy = ['name'];
            foreach ($orderBy as $order)
            {
                $designName = 'channels_by_iso_by_tag_by_' . $order;
                $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsByIsoByTag($designName, $order));
            }

            $designName = 'channels_playlists_by_iso_by_tag';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsPlaylistsByIsoByTag());

            $orderBy = ['date', 'name', 'position'];
            foreach ($orderBy as $order)
            {
                $designName = 'channels_by_iso_by_' . $order;
                $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsByIso($designName, $order));
            }

//            $designName = 'channels_by_iso';
//            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsByIso());
//
//            $designName = 'channels_by_iso_by_position';
//            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsByIsoByPosition());

            $designName = 'channels_id_by_iso';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsIdByIso());

            // Playlists
            $designName = 'channels_playlists';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsPlaylists());

            $designName = 'channels_playlists_by_id';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsPlaylistsById());

            $designName = 'channels_playlists_by_position';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsPlaylistsByPosition());

            $designName = 'channels_playlists_by_iso';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsPlaylistsByIso());

            $designName = 'channels_playlists_by_iso_by_position';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsPlaylistsByIsoByPosition());

            // Composes
            $designName = 'channels_composed_videos';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsComposedVideo());

            $orderBy = ['date', 'title', 'duration'];
            foreach ($orderBy as $order)
            {
                $designName = 'channels_composed_videos_by_iso_by_tag_by_' . $order;
                $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsComposedVideoByIsoByTag($designName, $order));
            }

            $orderBy = "doc.on[i][2]";
            $designName = 'channels_composed_videos_by_position';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsComposedVideo($designName, $orderBy));

            $orderBy = "doc.data.duration";
            $designName = 'channels_composed_videos_by_duration';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsComposedVideo($designName, $orderBy));

            $designName = 'channels_composed_videos_by_iso';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsComposedVideoByIso());

            $orderBy = "doc.on[i][2]";
            $designName = 'channels_composed_videos_by_iso_by_position';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsComposedVideoByIso($designName, $orderBy));

            $orderBy = "doc.data.profile.l10n[j]['title']";
            $designName = 'channels_composed_videos_by_iso_by_title';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsComposedVideoByIso($designName, $orderBy));

            $orderBy = "doc.data.duration";
            $designName = 'channels_composed_videos_by_iso_by_duration';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsComposedVideoByIso($designName, $orderBy));

            $designName = 'channels_playlists_id_by_iso';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsPlaylistsIdByIso());

            $designName = 'channels_playlists_composed_videos';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsPlaylistsComposedVideos($designName));

            $designName = 'channels_playlists_composed_videos_by_id';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsPlaylistsComposedVideoById());

            $orderBy = "doc.on[i][2]";
            $designName = 'channels_playlists_composed_videos_by_position';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsPlaylistsComposedVideos($designName, $orderBy));

            $orderBy = "doc.data.duration";
            $designName = 'channels_playlists_composed_videos_by_duration';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsPlaylistsComposedVideos($designName, $orderBy));

            $designName = 'channels_playlists_composed_videos_by_iso';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsPlaylistsComposedVideosByIso());

            $orderBy = "doc.on[i][2]";
            $designName = 'channels_playlists_composed_videos_by_iso_by_position';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsPlaylistsComposedVideosByIso($designName, $orderBy));

            $orderBy = "doc.data.profile.l10n[j]['title']";
            $designName = 'channels_playlists_composed_videos_by_iso_by_title';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsPlaylistsComposedVideosByIso($designName, $orderBy));

            $orderBy = "doc.data.duration";
            $designName = 'channels_playlists_composed_videos_by_iso_by_duration';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsPlaylistsComposedVideosByIso($designName, $orderBy));

            $orderBy = ['date', 'title', 'duration', 'position'];
            foreach ($orderBy as $order)
            {
                $designName = 'channels_playlists_composed_videos_by_iso_by_tag_by_' . $order;
                $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsPlaylistsComposedVideosByIsoByTag($designName, $order));
            }

            $designName = 'channels_playlists_composed_videos_by_id_by_iso';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ChannelsPlaylistsComposedVideosByIdByIso());

            $designName = 'latest_all';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\LatestAllVideos($designName));

            $designName = 'latest_all_by_iso';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\LatestAllVideosByIso($designName));

            $designName = 'latest_by_channel_by_tag';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\LatestVideosByChannelByTag($designName));

            $designName = 'latest_by_channel';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\LatestVideosByChannel($designName));

            $designName = 'latest';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\LatestVideos());

            $orderBy = "doc.on[i][2]";
            $designName = 'latest_by_position';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\LatestVideos($designName, $orderBy));

            $orderBy = "doc.data.duration";
            $designName = 'latest_by_duration';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\LatestVideos($designName, $orderBy));

            $orderBy = "doc.publish_at";
            $designName = 'latest_by_iso';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\LatestVideosByIso($designName, $orderBy));

            $orderBy = "doc.on[i][2]";
            $designName = 'latest_by_iso_by_position';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\LatestVideos($designName, $orderBy));

            $orderBy = "doc.data.profile.l10n[k]['title']";
            $designName = 'latest_by_iso_by_title';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\LatestVideos($designName, $orderBy));

            $orderBy = "doc.data.duration";
            $designName = 'latest_by_iso_by_duration';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\LatestVideos($designName, $orderBy));

            // Latest by channel by playlist by iso by tag order by publish date
            $orderBy = ['date', 'title', 'duration', 'position'];
            foreach ($orderBy as $order)
            {
//                $orderBy = "date";
                $designName = 'latest_by_iso_by_tag_by_' . $order;
                $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\LatestVideosByIsoByTag($designName, $orderBy));
            }

            $designName = 'live_by_date';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\LiveVideos($designName));

            $designName = 'live_by_iso_by_date';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\LiveVideosByIso($designName));

            $designName = 'live_videos_by_channel_by_tag';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\LiveVideosByChannelByTag($designName));

            $designName = 'lives_channel_by_tag';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\LivesChannelByTag($designName));

            $designName = 'live_videos_by_channel';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\LiveVideosByChannel($designName));

            $designName = 'composed_video_by_stickers_by_iso_by_position';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ComposedVideosByStickersByIsoByPosition($designName));

            $designName = 'composed_video_by_stickers_by_position';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ComposedVideosByStickersByPosition($designName));

            // GET /latest/channels
            $orderBy = ['title', 'duration', 'date'];
            foreach ($orderBy as $order)
            {
//                $orderBy = "date";
                $designName = 'latest_videos_by_iso_by_channeltag_by_' . $order;
                $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\LatestComposedVideoByIsoByChannelTag($designName, $orderBy));
            }

            $orderBy = ['duration', 'date'];
            foreach ($orderBy as $order)
            {
//                $orderBy = "date";
                $designName = 'latest_videos_by_channeltag_by_' . $order;
                $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\LatestComposedVideoByChannelTag($designName, $orderBy));
            }

            // GET /videos route
            $orderBy = ['duration', 'date'];
            foreach ($orderBy as $order)
            {
//                $orderBy = "date";
                $designName = 'videos_by_' . $order;
                $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ComposedVideos($designName, $orderBy));
            }

            $orderBy = ['title', 'duration', 'date'];
            foreach ($orderBy as $order)
            {
//                $orderBy = "date";
                $designName = 'videos_by_iso_by_tag_by_' . $order;
                $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ComposedVideosByIsoByTag($designName, $orderBy));
            }

            $orderBy = ['title', 'duration', 'date'];
            foreach ($orderBy as $order)
            {
//                $orderBy = "date";
                $designName = 'videos_by_iso_by_' . $order;
                $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ComposedVideosByIso($designName, $orderBy));
            }

            $designName = 'all_docs';
            $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\AllDocs());
        }
    }

}
