<?php namespace Engine\Job\Publish\Services\Videos\Schemas;

use DateTime;
use Engine\Job\Publish\Schema\Schema;
use Stage\Video\Channel\Channel;
use Stage\Account\Account;
use Stage\Account\Config\Config;
use Stage\Video\Playlist\Playlist;

class ChannelSchema extends Schema {

    /**
     * @var Channel
     */
    private $channel;

    /**
     * @var Config
     */
    private $config;

    /**
     * @var Account
     */
    private $account;

    public function __construct(Account $account, Config $config, Channel $channel, $registerDate, $updateDate)
    {
        $this->account = $account;
        $this->config = $config;
        $this->channel = $channel;
        $this->registerDate = (new DateTime($registerDate))->format(parent::DATETIME_FORMAT);
        $this->updateDate = !$updateDate ? null : (new DateTime($updateDate))->format(parent::DATETIME_FORMAT);
        $this->beginDate = (new DateTime($this->channel->getBeginDate()))->format(parent::DATETIME_FORMAT);
        $this->publishDate = (new DateTime($this->channel->getPublishDate()))->format(parent::DATETIME_FORMAT);
        $this->endDate = !$this->channel->getEndDate() ? null : (new DateTime($this->channel->getEndDate()))->format(parent::DATETIME_FORMAT);
        $this->assetBaseUrl = $config->get(Config::ASSET_IMAGE) . DIRECTORY_SEPARATOR . $account->getToken();

    }

    private function getAssetBaseUrl()
    {
        return $this->assetBaseUrl;
    }

    public function build()
    {
        $data = [
            'id' => $this->channel->getId(),
            'data' => [
                'id' => $this->channel->getId(),
                'l10n' => $this->l10n($this->channel),
                'domain' => $this->channel->getDomain(),
                'position' => $this->channel->getPosition(),
                'playlists' => [
                    'count' => $this->channel->getTotal()
                ],
                'date' => [
                    'begin' => $this->beginDate,
                    'end' => $this->endDate,
                    'register' => $this->registerDate,
                    'publish' => $this->publishDate,
                ],
            ],
            'publish_at' => $this->publishDate,
            'begin_at' => $this->beginDate,
            'end_at' => $this->endDate,
            'created_at' => $this->registerDate,
            'updated_at' => $this->updateDate,
            'type' => 'channel'
        ];

        // Set the default Playlist if exist
        if( $this->channel->getDefaultPlaylist() instanceof Playlist ) {
            $schema = new PlaylistSchema($this->account, $this->config, $this->channel->getDefaultPlaylist(), $this->channel->getDefaultPlaylist()->getRegisterDate(), null);
            $data['data']['playlist'] =  $schema->build()['data'];
            // $data['data']['playlist'] =  $this->channel->getDefaultPlaylist()->toArray();
        }

        // Get all the Channel languages
        $isos = $this->channel->innerLanguages();

        // Build the tags
        $data['tags'] = SchemaHelper::outterTags($isos, '\Engine\Job\Publish\Support\Video', 'channelTags', $this->channel->getId(), parent::DATETIME_FORMAT);

        // $data['channelTags'] = SchemaHelper::innerTags($this->channel->tags());

        return $data;
    }

    public function l10n(Channel $channel)
    {
        $l10ns = $channel->getL10n();
        return array_map(function($l10n) {
            $temp = $l10n->toArray();

            $temp2 = [];
            if( $temp['tags'] ) {
                foreach ($temp['tags'] as $tag) {

                    $t = [];
                    $t['id'] = $tag['id'];
                    $t['name'] = $tag['name'];
                    $t['slug'] = $tag['nameStripped'];
                    $t['count'] = $tag['total'];

                    $temp2['tags'][] = $t;
                }
            } else {
                $temp2['tags'] = [];
            }

            unset($temp['tags']);
            $temp['tags'] = $temp2['tags'];

            //$temp['isoLanguage'] = $temp['isoLanguage'];
            //$temp['created_at'] = (new DateTime($temp['registerDate']))->format(parent::DATETIME_FORMAT);
            $temp['date']['register'] = (new DateTime($temp['registerDate']))->format(parent::DATETIME_FORMAT);
            unset($temp['id']);
            unset($temp['registerDate']);
            //unset($temp['isoLanguage']);

            $std = new \stdClass();
            $std->file = $temp['image'];
            $std->baseUrl = $this->getAssetBaseUrl();
            $temp['image'] = $std;
            return $temp;
        }, $l10ns);
    }
}