<?php namespace Engine\Job\Publish\Services\Videos\Schemas;

use DateTime;
use Engine\Job\Publish\Schema\Schema;
use Stage\Account\Account;
use Stage\Account\Config\Config;
use Stage\Video\Playlist\Playlist;

class PlaylistSchema extends Schema {

    /**
     * @var Playlist
     */
    private $playlist;

    /**
     * @var Config
     */
    private $config;

    /**
     * @var Account
     */
    private $account;

    public function __construct(Account $account, Config $config, Playlist $playlist, $registerDate, $updateDate)
    {
        $this->account = $account;
        $this->config = $config;
        $this->playlist = $playlist;
        $this->beginDate = (new DateTime($this->playlist->getBeginDate()))->format(parent::DATETIME_FORMAT);
        $this->publishDate = !$this->playlist->getPublishDate() ? null : (new DateTime($this->playlist->getPublishDate()))->format(parent::DATETIME_FORMAT);
        $this->endDate = !$this->playlist->getEndDate() ? null : (new DateTime($this->playlist->getEndDate()))->format(parent::DATETIME_FORMAT);
        $this->registerDate = (new DateTime($registerDate))->format(parent::DATETIME_FORMAT);
        $this->updateDate =  !$updateDate ? null : (new DateTime($updateDate))->format(parent::DATETIME_FORMAT);
        $this->assetBaseUrl = $config->get(Config::ASSET_IMAGE) . DIRECTORY_SEPARATOR . $account->getToken();
    }

    private function getAssetBaseUrl()
    {
        return $this->assetBaseUrl;
    }

    public function build()
    {

        $data = [
            'id' => $this->playlist->getId(),
            'data' => [
                'id' => $this->playlist->getId(),
                // 'position' => $this->playlist->getPosition(),
                'l10n' => $this->l10n($this->playlist),
                'date' => [
                    'begin' => $this->beginDate,
                    'end' => $this->endDate,
                    'register' => $this->registerDate,
                    'publish' => $this->publishDate,
                ],
                'videos' => [
                    'count' => $this->playlist->totalComposes()
                ],
            ],
            'on' => $this->on(),
            'publish_at' => $this->publishDate,
            'begin_at' => $this->beginDate,
            'end_at' => $this->endDate,
            'created_at' => $this->registerDate,
            'updated_at' => $this->updateDate,
            'type' => 'playlist'
        ];

        // Get all the Playlist languages
        $isos = $this->playlist->innerLanguages();

        // $data['tags'] = SchemaHelper::outterTags($isos, $this->playlist->getId(), parent::DATETIME_FORMAT);
        // Build the tags
        $data['tags'] = SchemaHelper::outterTags($isos, '\Engine\Job\Publish\Support\Video', 'playlistTags', $this->playlist->getId(), parent::DATETIME_FORMAT);

        // $data['playlistTags'] = SchemaHelper::innerTags($this->playlist->tags());

        return $data;
    }

    public function l10n(Playlist $playlist)
    {
        $l10ns = $playlist->getL10n();
        return array_map(function($l10n) {
            $temp = $l10n->toArray();
            $temp2 = [];
            if( $temp['tags'] ) {
                foreach ($temp['tags'] as $tag) {

                    $t = [];
                    $t['id'] = $tag['id'];
                    $t['name'] = $tag['name'];
                    $t['slug'] = $tag['nameStripped'];
                    $t['count'] = $tag['total'];

                    $temp2['tags'][] = $t;
                }
            } else {
                $temp2['tags'] = [];
            }

            unset($temp['tags']);
            $temp['tags'] = $temp2['tags'];

            //$temp['isoLanguage'] = $temp['isoLanguage'];
            // $temp['created_at'] = (new DateTime($temp['registerDate']))->format(parent::DATETIME_FORMAT);
            $temp['date']['register'] = (new DateTime($temp['registerDate']))->format(parent::DATETIME_FORMAT);
            unset($temp['id']);
            unset($temp['registerDate']);
            //unset($temp['isoLanguage']);

            $std = new \stdClass();
            $std->file = $temp['image'];
            $std->baseUrl = $this->getAssetBaseUrl();
            $temp['image'] = $std;
            return $temp;
        }, $l10ns);
    }

    public function on()
    {
        $q = new \Stage\Video\Channel\Query();
        $q->from = 0;
        $q->to = -1;

        $channels = $this->playlist->channels();

        $temp = [];
        foreach ($channels as $channel)
        {
            $playlists = $channel->playlists($q);
            foreach ($playlists as $playlist)
            {
                if( $playlist->getId() == $this->playlist->getId() )
                {
                    $temp[] = [$channel->getId(), (int)$channel->getPosition(), (int) $playlist->getPosition()];
                }
            }
        }

        return $temp;
    }

}