<?php namespace Engine\Job\Publish\Support;

use Exception;
use PDO;
use Stage\Stage;

class Post
{

    public static function postsDossiersTags($idDossier, $isoLanguage)
    {
        try {

            $name = "POSTS_DOSSIERS_TAGS";
            $method = "GET";
            $parameters = [[$idDossier, PDO::PARAM_INT],
                           [$isoLanguage, PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);
            return $data;

        } catch(Exception $e){
            throw $e;
        }
    }

    public static function postsLatest($idCollection, $idDossier, $iso = "", $limit_a = 0, $limit_b = -1, $field = 0, $filter = "")
    {
        try {

            $name = "POSTS_LATEST";
            $method = "GET";
            $parameters = [[$idCollection, PDO::PARAM_INT],
                           [$idDossier, PDO::PARAM_INT],
                           [$iso, PDO::PARAM_STR],
                           [$limit_a, PDO::PARAM_INT],
                           [$limit_b, PDO::PARAM_INT],
                           [$field, PDO::PARAM_INT],
                           [$filter, PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);
            return $data;

        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * postsLatestCount
     *
     * @param $idCollection
     * @param $idDossier
     * @param string $iso
     * @return mixed
     * @throws Exception
     */
    public static function postsLatestCount($idCollection, $idDossier, $iso = "")
    {
        try {

            $name = "POSTS_LATEST_COUNT";
            $method = "GET";
            $parameters = [[$idCollection, PDO::PARAM_INT],
                           [$idDossier, PDO::PARAM_INT],
                           [$iso, PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);
            return $data[0]['total'];

        } catch(Exception $e) {
            throw $e;
        }
    }

    /**
     * @throws Exception
     */
    public static function publishPosts()
    {
        try {

            $name = "PUBLISH_POSTS";
            $method = "SET";
            $parameters = [];
            Stage::esp($parameters, $name, $method);

        } catch(Exception $e){
            throw $e;
        }

    }

    /**
     * @param $idCollection
     * @param $isoLanguage
     * @return array
     * @throws Exception
     */
    public static function postsAuthors($idCollection, $isoLanguage)
    {
        try {

            $name = "PUBLISH_POSTS_AUTHORS";
            $method = "GET";
            $parameters = [[$idCollection, PDO::PARAM_INT],
                           [$isoLanguage, PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);
            return $data;

        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * @param $idCollection
     * @param $isoLanguage
     * @return array
     * @throws Exception
     */
    public static function postsTags($idCollection, $isoLanguage)
    {
        try {

            $name = "PUBLISH_POSTS_TAGS";
            $method = "GET";
            $parameters = [[$idCollection, PDO::PARAM_INT],
                           [$isoLanguage, PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);
            return $data;

        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * postsToExecute
     *
     * @return array
     * @throws Exception
     */
    public static function postsToExecute()
    {
        try {

            $name = "PUBLISH_POSTS_TO_EXECUTE";
            $method = "GET";
            $parameters = [];

            $data = Stage::esp($parameters, $name, $method);
            return $data;

        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * postsTodaysLatest
     *
     * @return array
     * @throws Exception
     */
    public static function postsTodaysLatest($idCollection, $idDossier, $isoLanguage)
    {
        try {

            $name = "PUBLISH_POSTS_TODAYS_LATEST";
            $method = "GET";
            $parameters = [[$idCollection, PDO::PARAM_INT],
                           [$idDossier, PDO::PARAM_INT],
                           [$isoLanguage, PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);
            return $data;

        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * changed
     * Get all the objects to sync with couchdb
     * @return array
     * @throws Exception
     */
    public static function changed()
    {
        try {

            $name = "PUBLISH_POSTS";
            $method = "GET";
            $parameters = [];

            $data = Stage::esp($parameters, $name, $method);
            return $data;

        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * sync
     *
     * @param $id
     * @param $idSync
     * @param $type
     * @return array
     * @throws Exception
     */
    public static function sync($id, $idSync, $idSyncRev, $type)
    {
        try {

            $name = "PUBLISH_POSTS_SYNC";
            $method = "SET";
            $parameters = [[$id, PDO::PARAM_INT],
                           [$idSync, PDO::PARAM_STR],
                           [$idSyncRev, PDO::PARAM_STR],
                           [$type, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return $data;

        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * existIn
     *
     * Give all the pairs collection,dossier where the post exist
     * @param $idPost
     * @return array
     * @throws Exception
     */
    public static function existIn($idPost)
    {
        try {

            $name = "POST_DOSSIERS_AND_COLLECTIONS_IDS_RELATED";
            $method = "GET";
            $parameters = [[$idPost, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return $data;

        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * publishEnd
     *
     * @return array
     * @throws Exception
     */
    public static function publishEnd()
    {
        try {

            $name = "PUBLISH_POSTS_ENDS";
            $method = "SET";
            $parameters = [];

            $data = Stage::esp($parameters, $name, $method);
            return $data;

        } catch(Exception $e){
            throw $e;
        }
    }
}