<?php namespace Engine\Job\Publish\Support;

use Exception;
use PDO;
use Stage\Stage;
use Stage\Video\Sticker\Sticker;

class Video
{
    /**
     * videosLatest
     *
     * @param $idChannel
     * @param $idPlaylist
     * @return array
     * @throws Exception
     */
    public static function videosLatest($idChannel, $idPlaylist)
    {
        try {

            $name = "VIDEOS_LATEST";
            $method = "GET";
            $parameters = [[$idChannel, PDO::PARAM_INT],
                           [$idPlaylist, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return $data;

        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * channelTags
     *
     * @param $idChannel
     * @return array
     * @throws Exception
     */
    public static function channelTags($idChannel, $iso)
    {
        try {

            $name = "PUBLISH_VIDEO_TAGS";
            $method = "GET";
            $parameters = [[$idChannel, PDO::PARAM_INT],
                           [$iso, PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);
            return $data;

        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * playlistTags
     *
     * @param $idPlaylist
     * @param $iso
     * @return array
     * @throws Exception
     */
    public static function playlistTags($idPlaylist, $iso)
    {
        try {

            $name = "PUBLISH_PLAYLIST_TAGS";
            $method = "GET";
            $parameters = [[$idPlaylist, PDO::PARAM_INT],
                           [$iso, PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);
            return $data;

        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * publishVideos
     *
     * @throws Exception
     */
    public static function publishVideos($syncType)
    {
        try {

            $name = "PUBLISH_VIDEO";
            $method = "SET";
            $parameters = [[$syncType, PDO::PARAM_INT]];
            Stage::esp($parameters, $name, $method);

        } catch(Exception $e){
            throw $e;
        }

    }

    /**
     * videosToExecute
     *
     * @return array
     * @throws Exception
     */
    public static function videosToExecute()
    {
        try {

            $name = "PUBLISH_VIDEO_TO_EXECUTE";
            $method = "GET";
            $parameters = [];

            $data = Stage::esp($parameters, $name, $method);
            return $data;

        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * changed
     * Get all the objects to sync with couchdb
     * @return array
     * @throws Exception
     */
    public static function changed($syncType)
    {
        try {

            $name = "PUBLISH_VIDEO";
            $method = "GET";
            $parameters = [[$syncType, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return $data;

        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * sync
     *
     * @param $id
     * @param $idSync
     * @param $type
     * @return array
     * @throws Exception
     */
    public static function sync($id, $idSync, $idSyncRev, $type)
    {
        try {

            $name = "PUBLISH_VIDEO_SYNC";
            $method = "SET";
            $parameters = [[$id, PDO::PARAM_INT],
                            [$idSync, PDO::PARAM_STR],
                            [$idSyncRev, PDO::PARAM_STR],
                            [$type, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return $data;

        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * existIn
     *
     * Give all the pairs channel,playlist where the video composed exist
     * @param $idPost
     * @return array
     * @throws Exception
     */
    public static function existIn($idVideoComposed)
    {
        try {

            $name = "PUBLISH_VIDEO_PLAYLISTS_AND_CHANNELS_IDS_RELATED";
            $method = "GET";
            $parameters = [[$idVideoComposed, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return $data;

        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * publishEnd
     *
     * @return array
     * @throws Exception
     */
    public static function publishEnd($syncType)
    {
        try {

            $name = "PUBLISH_VIDEO_ENDS";
            $method = "SET";
            $parameters = [[$syncType, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return $data;

        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * stickersIn
     *
     * Get a compose stickers
     *
     * @param $idComposed
     * @return array
     * @throws Exception
     */
    public static function stickersIn($idComposed)
    {
        try {

            $parameters = [[$idComposed, PDO::PARAM_INT]];

            $name = "PUBLISH_VIDEO_STICKERS_BY_COMPOSED";

            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            $temp = [];
            foreach($data as $tag){
                $tagInstance = Sticker::createInstance($tag);
                array_push($temp, $tagInstance);
            }
            return $temp;

        } catch(Exception $e){
            throw $e;
        }
    }

    /**
     * @param $id
     * @param $query
     * @param $iso
     * @return array
     * @throws Exception
     */
    public static function videoSearch($id, $query, $iso)
    {
        try {

            $parameters = [[$id, PDO::PARAM_INT],
                           [$query, PDO::PARAM_STR],
                           [$iso, PDO::PARAM_STR]];

            $name = "PUBLISH_VIDEO_SEARCH";

            $method = "GET";
            return Stage::esp($parameters, $name, $method);

        } catch(Exception $e){
            throw $e;
        }
    }
}