<?php namespace Engine\Job\Publish;

use Exception;
use Illuminate\Http\Response;

class V2 extends Publish implements PublishInterface {

    /**
     * @var \Illuminate\Support\Facades\Log
     */
    private $logger;

    /**
     * @throws Exception
     */
    public function run(\Illuminate\Support\Facades\Log $logger = null)
    {
        $this->logger = $logger;

        $service = parent::getService();

        switch ($service)
        {
            case parent::POSTS:
                $this->posts();
                break;
            case parent::ASSETS:
                $this->assets();
                break;
            case parent::VIDEO:
                $this->videos(1);
                break;
            case parent::PLAYER:
                $this->videos(2);
                break;
            case parent::ADS:
                $this->ads();
                break;
        }
    }

    public function posts()
    {

        $fresh = false;

        try {

            \Engine\Job\Publish\Support\Post::publishPosts();

            $couch          = \Stage\Account\Settings\Couch::getInstance();
            $settings       = $couch->getSettings($this->getAccount());

            // Generate the database name
            $databaseName =  self::getServiceCouchDBName($this->getService(), $this->getAccount()->getId(), null);

            // Get the CouchDB handler
            $couchHandler = new Handlers\Couch\CouchHandler($settings->getHost(), $settings->getPort(), $databaseName);

            // Make a HEAD request to check database existence, otherwise create it
            $exist = $this->databaseExist($settings->getHost(), $settings->getPort(), $databaseName);

            // If return 404 means that database not exist
            if( $exist == Response::HTTP_NOT_FOUND )
            {
                $fresh = true;
                $couchHandler->getCouchDBClient()->createDatabase($databaseName);
            }

            // Create a new posts migration
            $migrate = new \Engine\Job\Publish\Services\Posts\Migrate($this->getAccount(), $this->getAccount()->getConfig());
            $migrate->setCouchHandler($couchHandler);
            $migrate->setOptimus($this->getOptimus());

            // Start migration
            $migrate->start($fresh);

        } catch(Exception $e) {
            throw $e;
        }

    }

    public function assets()
    {

        try {

            $couch          = \Stage\Account\Settings\Couch::getInstance();
            $settings       = $couch->getSettings($this->getAccount());

            // Create publish database if not exists
            $handler = $this->createPublishDatabase($settings);

            // Generate the database name (posts_idAccount_suffix)
            $response = $this->lastCount($handler, parent::ASSETS);

            if( $response->status == 404 )
            {
                $suffix = "_1";
            }
            else
            {
                $count = $response->body['count'];
                $currentCount = isset($count[parent::ASSETS]) ? $count[parent::ASSETS] : 0;
                $oldSuffix = "_". $currentCount;
                $currentCount ++;
                $suffix = "_" . $currentCount;
            }

            $database =  self::getServiceCouchDBName($this->getService(), $this->getAccount()->getId(), $suffix);

            // Get the Couch
            $couchHandler = new Handlers\Couch\CouchHandler($settings->getHost(), $settings->getPort(), $database);

            // Check if the database is created for this account
            try {

                $couchHandler->getCouchDBClient()->createDatabase($database);

            } catch (Exception $e) { }

            // Start publish to couch
            $migrate = new \Engine\Job\Publish\Services\Assets\Migrate($this->getAccount(),
                $this->getAlbums(),
                $this->getGalleries(),
                $this->getAssetsDirectories(),
                $this->getAssetBaseUrl(),
                $this->getAssets());
            $migrate->setCouchHandler($couchHandler);

            // Start
            $migrate->start();

            // Increment the service pubçish count on couchdb
            $this->incrementCount($handler, parent::ASSETS);

        } catch(Exception $e) {
            throw $e;
        }
    }

    public function videos($type)
    {

        $fresh = false;

        try {

            \Engine\Job\Publish\Support\Video::publishVideos($type);

            $couch          = \Stage\Account\Settings\Couch::getInstance();
            $settings       = $couch->getSettings($this->getAccount());

            // Generate the database name
            $databaseName =  self::getServiceCouchDBName($this->getService(), $this->getAccount()->getId(), null);

            // Get the CouchDB handler
            $couchHandler = new Handlers\Couch\CouchHandler($settings->getHost(), $settings->getPort(), $databaseName);

            // Make a HEAD request to check database existence, otherwise create it
            $exist = $this->databaseExist($settings->getHost(), $settings->getPort(), $databaseName);

            // If return 404 means that database not exist
            if( $exist == Response::HTTP_NOT_FOUND )
            {
                $fresh = true;
                $couchHandler->getCouchDBClient()->createDatabase($databaseName);
            }

            // Start publish to couch
            $migrate = new \Engine\Job\Publish\Services\Videos\Migrate($this->getAccount(), $this->getAccount()->getConfig(), $type);
            $migrate->setCouchHandler($couchHandler);
            $migrate->setLogger($this->logger);

            // Start
            $migrate->start($fresh);

        } catch(Exception $e) {
            throw $e;
        }
    }

    public function ads()
    {

        $fresh = false;

        try {

            \Engine\Job\Publish\Support\Ad::publishAds();

            $couch          = \Stage\Account\Settings\Couch::getInstance();
            $settings       = $couch->getSettings($this->getAccount());

            // Generate the database name
            $databaseName =  self::getServiceCouchDBName($this->getService(), $this->getAccount()->getId(), null);

            // Get the CouchDB handler
            $couchHandler = new Handlers\Couch\CouchHandler($settings->getHost(), $settings->getPort(), $databaseName);

            // Make a HEAD request to check database existence, otherwise create it
            $exist = $this->databaseExist($settings->getHost(), $settings->getPort(), $databaseName);

            // If return 404 means that database not exist
            if( $exist == Response::HTTP_NOT_FOUND )
            {
                $fresh = true;
                $couchHandler->getCouchDBClient()->createDatabase($databaseName);
            }

            // Create a new posts migration
            $migrate = new \Engine\Job\Publish\Services\Ads\Migrate($this->getAccount(), $this->getAccount()->getConfig());
            $migrate->setCouchHandler($couchHandler);

            // Start migration
            $migrate->start($fresh);

        } catch(Exception $e) {
            throw $e;
        }
    }

    public function createPublishDatabase($settings)
    {
        $database = 'publish';

        // Get the Couch
        $couchHandler = new Handlers\Couch\CouchHandler($settings->getHost(), $settings->getPort(), $database);

        try {

            // Create database if not exists
            $couchHandler->getCouchDBClient()->createDatabase($database);

        } catch (Exception $e) { }

        return $couchHandler;
    }

    public function lastCount($couchHandler)
    {

        $doc = "account_".$this->getAccount()->getId();

        $rev = null;
        try {

            $document = $couchHandler->getCouchDBClient()->findDocument($doc);

//            if( $document->status == 200 )
//                $rev = $document->body['_rev'];

            // The rev is null in case of a not found document
            return $document;

        } catch (Exception $e) { }

    }

    public function incrementCount($couchHandler, $service)
    {

        $doc = "account_" . $this->getAccount()->getId();

        $rev = null;
        $response = $this->lastCount($couchHandler, "");
        if( $response->status == 200 )
        {
            // Get the rev
            $rev = $response->body['_rev'];

            // Increment the count
            $count = $response->body['count'];
            $currentCount = isset($count[$service]) ? $count[$service] : 0;
            $currentCount ++;

            $data = array_merge($count, [$service => $currentCount]);

        }
        else
        {
            $data = [
                parent::POSTS => 1,
                parent::ASSETS => 1,
                parent::VIDEO => 1,
                parent::PLAYER => 1
            ];
        }

        // Save on couch
        // "_id" => $this->getAccount()->getId()
        $data = ["count" => $data,
            "at" => $this->at()];

        return $couchHandler->getCouchDBClient()->putDocument($data, $doc, $rev);
    }

    public function at()
    {
        return (new \DateTime())->format(DATE_ATOM);
    }

    /**
     * @param $host
     * @param $port
     * @param string $name
     */
    public function databaseExist($host, $port, $name = '')
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, 'http://'.$host.':'.$port.'/' . $name);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'HEAD');
        curl_setopt($ch, CURLOPT_NOBODY, true);

        $result = curl_exec($ch);
        $httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);  // 200 || 404
//            if (curl_errno($ch))
//            {
//                echo 'Error:' . curl_error($ch);
//            }
        curl_close ($ch);

        return $httpcode;
    }
}