<?php

use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Engine\Job\Publish\Publish;
use Stage\StoredProcedure\Manager as StoredProcedureManager;

class StageTest extends \PHPUnit_Framework_TestCase {

	public function setUp()
	{
		/*$credentials = [
			'host' => "10.102.2.22",
			'dbname' => "StageAccount",
			'username' => "yodeveloper",
			'password' => "qwer123"
		];*/

        $credentials = [
            'host' => "192.168.70.21",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "192.168.70.21",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

		$this->stage = Stage::getInstance();

		// Create a default connection
		$default = new Connection($credentials);
        $client = new Connection($clientCredentials);

		// Add connection to the stage
		$this->stage->addConnection('default', $default);
        $this->stage->addConnection('client', $client);

        // Set the Stage stored procedures
        $this->stage->setSps(Sps::load());

		// Set the StoredProcedure\Manager instance
		$spm = StoredProcedureManager::getInstance();
		$this->stage->setStoredProcedureManager($spm);

	}

	public function tearDown()
	{

	}

	public function testIsInstance()
	{
		$this->assertInstanceOf('Stage\Stage', $this->stage);
	}

	public function testDateTime()
    {
        date_default_timezone_set("Europe/Lisbon");
        print_r((new DateTime("2019-10-28 15:15:15"))->format(DateTime::ATOM));
        $d = DateTime::createFromFormat(DateTime::ATOM, "2019-04-02T11:19:05+04:00");
        echo $d->format(DateTime::ATOM);
        $dateAux = new DateTime("2019-04-02T11:19:05+00:00");
        print_r($d);
    }

	public function testDateFormat()
    {
        $format = "Y/m/d H:i:s O";
        $mysqlDateTime = "2019-01-21 15:19:03";
        $new = (new DateTime($mysqlDateTime))->format($format);
        $this->assertEquals("2019/01/21 15:19:03 +0000", $new);

        // D, d M Y H:i:s O
        $datetime = new DateTime();
        $formated = $datetime->format($format);
    }

	public function testEnginesPublishPosts()
    {
        $account        = \Stage\Account\Account::find(3);

        $couch          = \Stage\Account\Settings\Couch::getInstance();
        $settings       = $couch->getSettings($account);

        $database = "posts";
        $couchHandler = new \Engine\Job\Publish\Handlers\Couch\CouchHandler($settings->getHost(), $settings->getPort(), $database);
        //$couchHandler->getCouchDBClient()->createDatabase($database);

        // $migrate = new \Engine\Job\Publish\Services\Posts\Migrate($account, $this->getCollections(), $this->getAssetBaseUrl());
        // $migrate->setCouchHandler($couchHandler);

        // print_r($couchHandler->getCouchDBClient());

        // Create a new Engine
        $engine = new Engine\Engine();

        // Posts publish
        $job1 = new Engine\Job\Publish\V2($account, Publish::POSTS);

        // Add the jobs to the queue
        $engine->addJob($job1);

        // Run the engine jobs
        $engine->run();

        // Actualiza a data do publish end date
        \Engine\Job\Publish\Support\Post::publishEnd();
    }

    public function testOptimus()
    {
        $prime = 1580030173;
        $inverse = 59260789;
        $xor = 1163945558;
        $size = 33;
        $inst = new \Stage\Optimus($prime, $inverse, $xor, $size);
        $inst->setMode(\Stage\Optimus::MODE_GMP);

        $t = [];
        for($i = 0 ; $i <= 30000 ; $i++)
        {
            $packed = \Stage\Helper\Video\Hash::pack([0, 0, $i]);
            $hash = $inst->encode($packed);
            if( in_array($hash, $t) )
            {
                echo $packed . "=>" . $hash . "\n";
            }

            $t[] = $hash;
        }
    }

    public function testEnginesPublishAds()
    {
        $account        = \Stage\Account\Account::find(3);

        // Create a new Engine
        $engine = new Engine\Engine();

        // Posts publish
        $job1 = new Engine\Job\Publish\V2($account, Publish::ADS);

        // Add the jobs to the queue
        $engine->addJob($job1);

        // Run the engine jobs
        $engine->run();
    }

    public function testEnginesPublishVideos()
    {
        $account = \Stage\Account\Account::find(3);

        $clientCredentials = [
            'host' => "192.168.70.21",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        // Reset all the fields
        $dsn = sprintf("mysql:host=%s;dbname=%s", $clientCredentials['host'], $clientCredentials['dbname']);
        $dbh = new PDO($dsn, $clientCredentials['username'], $clientCredentials['password']);
        $dbh->query("UPDATE video_composed SET id_sync = '', id_rev_sync = '', state=5;");
        $dbh->query("UPDATE video_composed SET id_sync_player = '', id_rev_sync_player = '', state=5;");
        $dbh->query("UPDATE channels SET id_sync = '',id_rev_sync = '', state=5;");
        $dbh->query("UPDATE playlists SET id_sync = '',id_rev_sync = '', state=5;");

        // Delete couchdb
        $couch          = \Stage\Account\Settings\Couch::getInstance();
        $settings       = $couch->getSettings($account);

        // Generate the database name
        $databaes = ["video_" . $account->getId(), "players_" . $account->getId()];
        foreach($databaes as $databaseName)
        {
            // Get the CouchDB handler
            $couchHandler = new \Engine\Job\Publish\Handlers\Couch\CouchHandler($settings->getHost(), $settings->getPort(), $databaseName);

            $couchHandler->getCouchDBClient()->deleteDatabase($databaseName);
        }

        // Create a new Engine
        $engine = new Engine\Engine();

        // Posts publish
        $job1 = new Engine\Job\Publish\V2($account, Publish::VIDEO);

        // Add the jobs to the queue
        $engine->addJob ($job1);

        // Run the engine jobs
        $engine->run();

    }

    public function testEnginesPublishPlayers()
    {
        $account = \Stage\Account\Account::find(3);

        // Create a new Engine
        $engine = new Engine\Engine();

        // Posts publish
        $job1 = new Engine\Job\Publish\V2($account, Publish::PLAYER);

        // Add the jobs to the queue
        $engine->addJob($job1);

        // Run the engine jobs
        $engine->run();
    }

//    public function testEnginesPublishLive()
//    {
//        $account = \Stage\Account\Account::find(3);
//
//        // Create a new Engine
//        $engine = new Engine\Engine();
//
//        // Posts publish
//        $job1 = new Engine\Job\Publish\V2($account, Publish::LIVE);
//
//        // Add the jobs to the queue
//        $engine->addJob($job1);
//
//        // Run the engine jobs
//        $engine->run();
//    }

    public function testEnginesPublishFactory()
    {
        // Create a new Engine
        $engine = new Engine\Engine();        // Create a new Engine
        $engine = new Engine\Engine();

//        $parserFactory = new \DesignPatterns\Creational\AbstractFactory\ParserFactory();
//
//        $jsonParser = $parserFactory->createJsonParser();
//        $xmlParser = $parserFactory->createXmlParser('someOption');

        $publishFactory = new PublishFactory();
        $publishV2 = $publishFactory->createPublish($version);

    }

	public function testBenchmarkCouchDb()
    {

        // Start of execution
        $start = date_create();

        // 1. Connect to couchDB
        // $account        = \Stage\Account\Account::find(3);

        // $couch          = \Stage\Account\Settings\Couch::getInstance();
        // $settings       = $couch->getSettings($this->getAccount());

        $couchHost = "10.102.2.30";
        $couchPort = "5984";
        $database = 'benchmark_' . date('d_m_Y_H_i_s');
        $couchHandler = new \Engine\Job\Publish\Handlers\Couch\CouchHandler($couchHost, $couchPort, $database);
        $couchHandler->getCouchDBClient()->createDatabase($database);

        // 2. Insert some data
        // $iterations = dgetenv('iterations') || 2;
        $iterations = 2;
        $temp = [];
        for ( $i = 0 ; $i <= $iterations ; $i++ )
        {
            $data = $this->data();
            $iso = 'pt';
            $payload = ["data" => $data, "type" => "post", "iso" => $iso];
            $temp[] = $payload;
//            $couchHandler->getCouchDBClient()->postDocument(["data" => $data, "type" => "post", "iso" => $iso]);
        }


        $bulkUpdater2 = $couchHandler->getCouchDBClient()->createBulkUpdater();
        $bulkUpdater2->updateDocuments($temp);
        $response = $bulkUpdater2->execute();

        // End of execution
        $end = date_create();

        // Calculate the estimate execution time
        $eet  	= date_diff( $start, $end );

        $data = $eet->format("%H:%I:%S");
        $couchHandler->getCouchDBClient()->postDocument(["id" => 'eet', "data" => $data, "type" => "post", "iso" => $iso]);

        $client = $couchHandler->getCouchDBClient();
        $designName = 'collections';

        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\Collections());

        $designName = 'collections_dossiers';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\CollectionsDossiers());

        $designName = 'collections_dossier';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\CollectionsDossier());

        $designName = 'collections_dossiers_post';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\CollectionsDossiersPost());

        $designName = 'collections_dossiers_posts';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\CollectionsDossiersPosts());

        $designName = 'latest';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\LatestPosts());

        $designName = 'all_docs';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\AllDocs());

        $designName = 'etag';
        $client->createDesignDocument($designName, new \Engine\Job\Publish\Handlers\Couch\DesignDocuments\ETag());


        echo "\nEET: " . $eet->format("%H:%I:%S") . "\n";

    }

}